/** enero 12 1997        get dipeptide composition from sequence file  **/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#define ID     0
#define SQ     1
#define SLASH  2

FILE *infile,*outfile;
FILE *SEQerror;

static char buf [256];
static char IDnum [10];
static char curIDSTR[33];

#define AATABLEN 20
float         statistic [AATABLEN][AATABLEN];
unsigned long totalComp [AATABLEN][AATABLEN];


void NormalString(char *);
void SeqSum    (char *);
int FindTable (char );
char * Filter (char *);

void infoexit (char *);

unsigned seqLen,curSQLEN=0;
int curSQunormalNUM;



int FindTable(char c)
{
	static char aaTAB[]="CSTPAGNDEQHRKMILVFYW";
	int i;

	for (i=0;i<AATABLEN;i++)
		 if (c==aaTAB[i]) break;
	if  (i==AATABLEN) return -1;
	else return i;
}

void ReadStatisticMatrix (void)
{
   int i;
   float         *pm;
   unsigned long *pm1;

   pm  = (float *)&statistic[0][0];
   pm1 = (unsigned long *)&totalComp[0][0];

	  for (i=0;i<AATABLEN*AATABLEN;i++,pm++)
		   *pm=0.0;

	  for (i=0;i<AATABLEN*AATABLEN;i++,pm1++)
		   *pm1=0;
}

void WriteStatisticMatrix (void)
{
	 int i,j;

	 fseek(outfile,0L,SEEK_CUR);
	 fprintf(outfile,"%s\n",curIDSTR);

	 for (i=0;i<AATABLEN;i++){
		  for (j=0;j<AATABLEN;j++)
			   fprintf(outfile,"%8.4f", statistic[i][j]*100/seqLen);

		  fprintf(outfile,"\n");
	 }
}


void WriteTotalComp (void)
{
	 int i,j;

	 fseek(outfile,0L,SEEK_CUR);

	 fprintf(outfile,"\n");
	 fprintf(outfile,"The dipeptide composition for this sequence file is:\n");

	 for (i=0;i<AATABLEN;i++){
		  for (j=0;j<AATABLEN;j++)
			   fprintf(outfile,"%4u",totalComp[i][j]);
		  fprintf(outfile,"\n");
	 }
}


void NormalString(char *str)
{
	 char *p;
	 int  i,j;

	 for (p = str;*p;++p)
		  if(isalpha(*p))  {
			 *str = toupper(*p);
			 ++str;
		  }
	 *str=0;
}


char * Filter (char *str)
{
	while (*str== ' ' ||*str==0x9) str++;
	return str;
}


int analyses (char *buf)
{
	if (buf[0]=='I' && buf[1]=='D')
	   return ID;
	else if (buf[0]=='S' && buf[1]=='Q')
	   return SQ;
	else if (buf[0]=='/' && buf[1]=='/')
	   return SLASH;
	else return -1;
}

void SeqSum (char *sq)
{
	int i,len;
	static char lastAA=0;
	int lastUnormalAAnum;

	char *p;
	int x,y;

	p=sq;
	lastUnormalAAnum = 0;
	len = strlen(sq);
	curSQLEN -=len;
	if (lastAA) {
		x = FindTable(lastAA);
		y = FindTable(*sq);
		if ((x!=-1)&&(y!=-1)) {
			 statistic[x][y] +=1.0;
			 totalComp[x][y] +=1.0;
		}
		else lastUnormalAAnum ++;
	}

	x = FindTable(*sq);
	if (x!=-1) i=0;
	else {
		   i=1;
		   lastUnormalAAnum ++;
	}
	for (; i<(len-1); i++,sq++,x=y){
		 y = FindTable(*(sq+1));
		 if ((x!=-1)&&(y!=-1)) {
			 statistic[x][y] +=1.0;
			 totalComp[x][y] +=1.0;
		 }
		 else lastUnormalAAnum ++;
	}

	if (curSQLEN) lastAA=*sq;
	else lastAA=0;

	if (lastUnormalAAnum!=0) {
		  if (curSQunormalNUM==0)
			  fprintf(SEQerror,"%s:\n",curIDSTR);
		  fprintf(SEQerror,"\t%s\n",p);
		  curSQunormalNUM+=lastUnormalAAnum;
	}
}


void infoexit(char *msg)
{
	 printf("\n",msg);
	 exit (0);
}

void help()
{
	 printf("Usage: DIPEPCOM <source file> <target file>\n");
	 printf("The source file should be a sequence file from the SWISS-PROT.\n");
	 printf("The target file is the dipeptide compostion of those sequences.\n");
	 exit(0);
}


void main (int argc, char *argv[])
{
   int i,j;
   int processID;
   unsigned long CurSQno=0;  /** current sequence number  **/
   char id[20];

   system ("clear");
   if (argc!=3) help();

   if ( (infile=fopen(argv[1],"r")) == NULL )
	 infoexit("Source file not found.");
   if ( (outfile=fopen(argv[2],"w")) == NULL )
	 infoexit("Target file cannot created.");

   if ( (SEQerror=fopen("seq.err","w")) == NULL )
	 infoexit("sequence error report file not created.");


   ReadStatisticMatrix ();
   fprintf (outfile,"          \n");

   processID=ID;

   while  (fgets(buf,255,infile)!=NULL) {

	 switch (analyses (buf)){

	   case ID:
		 if (processID==ID) {
			 CurSQno++;

			 /** for the begining of a new sequence, the
sequence length should be 0   **/
			 if (curSQLEN)
			   infoexit("last sequence data has not been finished.");


			 sscanf (buf,"%s%s",id,id);
			 strcpy (curIDSTR,id);

			 processID=SQ;

			 printf("processing sequence no.");
			 printf("%u\n",CurSQno);
		 }
		 else infoexit("being waiting for a ID");
		 break;

		 case SQ:
		 if (processID==SQ)  {

			 sscanf (buf, "%s%s%u",id,id,&curSQLEN);
			 seqLen = curSQLEN;
			 processID = SLASH;

			 curSQunormalNUM = 0;

		 }
		 else infoexit("being waiting for a SQ");
		 break;

	   case SLASH:

		 if (processID==SLASH) {
			 WriteStatisticMatrix ();
			 processID=ID;
			 for (i=0;i<AATABLEN;i++)
				 for (j=0;j<AATABLEN;j++)
					  statistic[i][j] = 0;
		 }
		 else infoexit("being waiting for two slashes.");
		 break;

	   case -1:
		 if (processID==SLASH) {
			 NormalString(buf);

			 SeqSum(buf);
		 }
		 break;
	   }

	 }

/** write down the dipeptide composition of all database sequence **/

	 WriteTotalComp();
	 rewind  (outfile);
	 fprintf (outfile,"%u",CurSQno);
	 printf  ("======== FINISHED ========\n");

	 fclose  (infile);
	 fclose  (outfile);
	 fclose  (SEQerror);
}
