/** for sequence input, both interleaved and sequential
(phylip) format can be used      **/

#include <stdio.h>
#include <string.h>
#include "sota.h"

void ** new_matrix( int rows, int cols, int size )
{
	char **p;
	int i;

	p = (char **) malloc( rows*sizeof( char *) );
	if ( p==NULL ) {
		printf("No more memory!\n");
		exit(1);
	}
	for( i=0; i<rows; i++ ) {
		p[i] = (char *) malloc( cols*size );
		if ( p==NULL ) {
			printf("No more memory!\n");
			exit(1);
		}
	}
	return( (void **)p );
}

/* 18-4-95
=============================================================================
MODULO: INPUT.NNT

DESCRIPCION : Contiene las funciones para la toma de datos.

=============================================================================
*/
/*-----------------------------
LEE BASE:
-----------------------------*/

void lee_residuo( int j, int i, char b, DATA *D )
{
	int k, ind, flag=0;
	real **sec;

	sec = D->inputvector[i];

	for ( k=0; k < D->tamalfabeto; k++ )  sec[j][k] = 0;

	if( D->residuoposible == Base ) {
		for( ind=0; ind<4; ind++ ) {
			if( BasePosible[ind]==b ) {
				flag = 1;
				break;
			}
		}
		if ( flag ) sec[j][ind] = 1.0;
		else {
			for( ind=0; ind<4; ind++ )  sec[j][ind] = 0.25;
		}
	}
	else {
		for( ind=0; ind<22; ind++ ) {
			if( AacPosible[ind]==b ) {
				flag = 1;
				break;
			}
		}
		if( flag ) sec[j][ind] = 1.0;
		else {
			for( k=0; k<D->tamalfabeto; k++ )
				sec[j][k] = 1.0/D->tamalfabeto;
		}
	}
}


/* -----------------------------
INPUT DATA:
----------------------------- */

int seqinputvals( DATA *Datos )
{
	char line[100];

	if ( (Datos->fp = fopen( Datos->namein, "r" ))==NULL ) return( FALSE );
	fgets( line, 100, Datos->fp );
	if ( sscanf( line, "%d %d", &Datos->numseq, &Datos->numbases )==2 )
		return( TRUE );
	return( FALSE );
}

void seqinputdata( DATA *Datos )
{
	int i, j, size, totsize;
	int linesize=0;      /* residue number in a sequence line */
	char residuo;
	char line[128];
	char name[ 20 ], aux[118], *paux;

	switch (Datos->inputseqformat) {
		case interleaved:
			for( i=0; i < Datos->numseq; i++ ) {
				if ( fgets( line, 100, Datos->fp ) == NULL ) {
					printf("No sequence!\n");
					exit( 1 );
				}
				Datos->inputvector[i] =
					(real **)new_matrix( Datos->numbases, Datos->tamalfabeto, sizeof(real) );

				for( j=0; j<10; j++ ) name[j] = line[j];
				name[10] = 0;
				/** nombre de secuencia **/
				j = strlen( name );
				Datos->nombre[i] = (char *) malloc( j+1 );
				strcpy( Datos->nombre[i], name );
				if ( Datos->debug ) printf("%d Leida %s\n", i, name );

				/**  la secuencia  **/
				for( paux=line+10; *paux==' ' || *paux=='\t' || isdigit(*paux); paux++ );
				for( totsize=0; totsize<Datos->numbases; ) {
					while ( *paux==' ' || *paux=='\t' || isdigit(*paux) ) paux++;
					if ( *paux=='\n' ) break;

					residuo = toupper( *paux );
					lee_residuo( totsize, i, residuo, Datos );
					totsize++;
					paux++;
				}
			}
			if (totsize == Datos->numbases) break; /* each sequence is in one line */

			do {
				for( i=0; i < Datos->numseq; i++ ) {
					if (i != 0) totsize -= linesize;
					linesize = 0;
					if ( fgets( line, 100, Datos->fp ) == NULL ) {
							printf("sequence length is less than the argument or mismatch sequence format!\n");
							exit( 1 );
					}
					paux=line;
					if (*paux == '\n') {
						if ( fgets( line, 100, Datos->fp ) == NULL ) {
							printf("sequence length is less than the argument or mismatch sequence format!\n");
							exit( 1 );
						}
					}
					for( paux=line; *paux==' ' || *paux=='\t' || isdigit(*paux); paux++ );
					for( j=0;totsize<Datos->numbases;) {
						while( *paux==' ' || *paux=='\t' || isdigit(*paux) ) paux++;
						if ( *paux=='\n' ) break;
						residuo = toupper( *paux );
						residuo = toupper( *paux );
						lee_residuo( totsize, i, residuo, Datos );
						totsize++;
						linesize++;
						paux++;
					}
				}
			} while(totsize < Datos->numbases);
			break;

		case sequential:
			for( i=0; i < Datos->numseq; i++ ) {
				if ( fgets( line, 100, Datos->fp ) == NULL ) {
					printf("No sequence line!\n");
					exit( 1 );
				}
				Datos->inputvector[i] =
					(real **)new_matrix( Datos->numbases, Datos->tamalfabeto, sizeof(real) );

				for( j=0; j<10; j++ ) name[j] = line[j];
				name[10] = 0;
				/** nombre de secuencia **/
				j = strlen( name );
				Datos->nombre[i] = (char *) malloc( j+1 );
				strcpy( Datos->nombre[i], name );
				if ( Datos->debug ) printf("%d Leida %s\n", i, name );

				/** la secuencia  **/
				for( paux=line+10; *paux==' ' || *paux=='\t' || isdigit(*paux); paux++ );
				for( totsize=0; totsize < Datos->numbases;  ) {
					while ( *paux==' ' || *paux=='\t' || isdigit(*paux) )paux++;
					if ( *paux=='\n' ) break;
					residuo = toupper( *paux );
					lee_residuo( totsize, i, residuo, Datos );
					totsize++;
					paux++;
				}
				do {
					if ( totsize < Datos->numbases ) {
						if ( fgets( line, 100, Datos->fp ) == NULL ) {
							printf("sequence length is less than the argument or mismatch sequence format!\n");
							exit( 1 );
						}
						for( paux=line; *paux==' ' || *paux=='\t' || isdigit(*paux); paux++ );
						for( j=0; totsize < Datos->numbases;  ) {
							while( *paux==' ' || *paux=='\t' || isdigit(*paux) )paux++;
							if ( *paux=='\n' ) break;
							residuo = toupper( *paux );
							lee_residuo( totsize, i, residuo, Datos );
							totsize++;
							paux++;
						}
					}
					else break;
				} while(totsize < Datos->numbases);

			}
			break;
		}

		fclose ( Datos->fp );
		Datos->fp = NULL;
}

