/** 29-12-95 { 13-9-95 Pascal version } **/
/** Last update 10-03-1997  **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "sota.h"

#define NumBaseP    5
#define NumAacP    24

char version[] = "3.0";
char BasePosible[NumBaseP+1] ="AGCT-";
/**                           0        90        9              **/
char AacPosible[NumAacP+1]  ="ACDEFGHIKLMNPQRSTVWY-.?X";
char ResidIndet[3] = "?X";          /** Indeterminations **/
char ResidDel[3] = "-.";            /** deletions **/

float cputime_in_minute;

/* pseudorandom number generator */
float ran0( int * idum )
{
	static float y,maxran,v[98];
	unsigned int semilla;
	static int iff=0;
	int j;

	if ( *idum <0 || iff == 0) {
		semilla = (unsigned int) time ( NULL );
		srand( semilla );
		iff=1;
		*idum=1;
		maxran = RAND_MAX + 1.0;
		for (j=1;j<=97;j++) v[j]=rand();
		y=rand();
	}
	j= 1 + (int)( 97.0f*y/maxran );
/** 	if (j > 97 || j < 1) printf("RAN0: This cannot happen.\n");  **/
	y=v[j];
	v[j]=rand();
	return (y/maxran);
}


/* Create a new node of the tree structure */
PCell  new_node( int rows, int cols )
{
	PCell p;

	p = (PCell) malloc( sizeof( Cell ) );
	if ( p != NULL )
		p->sec=(real **)new_matrix( rows, cols, sizeof(real) );
	if ( p==NULL || p->sec==NULL ) {
		printf("No more memory!!!\n");
		exit(1);
	}
	return ( p );
}


/*---------------------------------------
INITIALIZE CELL:
---------------------------------------*/

void iniciacell ( PCell  c, int rows, int cols )
{
	int i, j;
	real suma, **s;
	int seed=-4;

	s = c->sec;
	
	for ( i=0; i < rows; i++ ) {
		for ( j=0; j < cols; j++ ) s[i][j] = ran0( &seed );
		/** Normalizar para que las proporciones de cada base sumen 1 **/
		suma = 0;
		for ( j=0; j < cols; j++ )  suma +=  s[i][j];
		for ( j=0; j < cols; j++ )  s[i][j] /= suma;
	}
	c->up=NULL; c->rig=NULL; c->lef=NULL;
	c->id = -1;      /** not initialized **/
	c->update = TRUE;
}


/*---------------------------------------
INITIALIZE TREE:

              ROOT
			 //  \\
        node        node
          ^           ^
  list  [ 0 ]       [ 1 ]
---------------------------------------*/

void inicia_arbol( DATA *D  )
{
	int i, rows, cols;
	PCell p;

	rows = D->numbases;;
	cols = D->tamalfabeto;

	p = new_node ( rows, cols );
	iniciacell( p, rows, cols );

	p->rig = new_node( rows, cols );
	iniciacell( p->rig, rows, cols ); 
	p->rig->up = p;

	p->lef = new_node( rows, cols );
	iniciacell( p->lef, rows, cols ); 
	p->lef->up = p;

	D->lcells.num = 2;
	D->lcells.cells[0] = p->lef;
	D->lcells.cells[1] = p->rig;

	/** indexes for identification within the tree **/
	p -> id = 0;
	p -> lef -> id = 1;
	p -> rig -> id = 2;

	p -> lef -> indlist = 0;
	p -> rig -> indlist = 1;

	for( i=0; i < D->numseq; i++ ) D->lcells.rango[i] = p;

	D->raiz = p;
}


/*-----------------------------------------------------
DUPLICATE NODE i:
	nodei                   nodei
	  ^                    //  \\
	  |    ==>         nodei1   nodei2
	  |                 ^           ^
    lista[i]        lista [ i ]  lista[ lcells.NumCells+1 ]
--------------------------------------------------------*/

void duplica_nodo( int *indi, DATA *D )
{
	int j, h, k;
	int i;

	i = D->indice;
	 
	/** Each sequence that fell in node i       **/
	/** are assigned to the range of influence  **/
	for ( j=0; j < D->numseq; j++ )
		if ( D->inputBMU[j] == i )  D->lcells.rango[j] = D->lcells.cells[i];

	/** Disable actualization in mother node **/
	D->lcells.cells[i]->up->update = FALSE;

	/** Create left descendant, copy the sequence and point to each other **/
	D->lcells.cells[i]->lef = new_node( D->numbases, D->tamalfabeto  );
	for( h=0; h<D->numbases; h++ ) {
		for( k=0; k<D->tamalfabeto; k++ ) {
			D->lcells.cells[i]->lef->sec[h][k] = D->lcells.cells[i]->sec[h][k];
		}
	}
	D->lcells.cells[i]->lef->up  = D->lcells.cells[i];
	D->lcells.cells[i]->lef->rig = NULL;
	D->lcells.cells[i]->lef->lef = NULL;
	D->lcells.cells[i]->lef->id = *indi;
	++(*indi);

	/** the same is done with right descendent  **/
	D->lcells.cells[i]->rig = new_node(D->numbases, D->tamalfabeto );
	for( h=0; h<D->numbases; h++ ) {
		for( k=0; k<D->tamalfabeto; k++ ) {
			D->lcells.cells[i]->rig->sec[h][k] = D->lcells.cells[i]->sec[h][k];
		}
	}
	D->lcells.cells[i]->rig->up  = D->lcells.cells[i];
	D->lcells.cells[i]->rig->rig = NULL;
	D->lcells.cells[i]->rig->lef = NULL;

	D->lcells.cells[i]->rig->id = *indi;
	++(*indi);

	/** for the right one a new pointer is created **/
	/** it is pointed from lista i, which in this  **/
        /** moment is pointing to the node **/
	D->lcells.cells[ D->lcells.num ] = D->lcells.cells[i]->rig;
	D->lcells.cells[ D->lcells.num ]->indlist = D->lcells.num;

	/** cells number is increased **/
	++D->lcells.num;

	/** Left descendant is pointed by the old index **/
	D->lcells.cells[i] = D->lcells.cells[i]->lef;
	D->lcells.cells[i]-> indlist = i;     /** old index **/
}


/* ---------------------------------------
DISTANCIA:
--------------------------------------- */

real distancia( char modea, real **x, real **w, int bases, int tam )
{
	real d, d1, sj;
	int i, j;

	d = 0;
	if (modea == '1') {
		for( i=0; i<bases; i++ ) {
			sj = 0;
			for ( j=0; j<tam; j++ )  sj += x[i][j] * w[i][j];
			if ( sj > 1.0f ) sj = 1.0f;  /* in the beggining it could happen */
			d += 1.0f - sj;
		}
	/** future Jukes & Cantor, etc. corrections **/
		return( 10*d / bases );
	}

	for ( i=0; i<bases; i++) {        /** if modea == '2' or '3' **/
		for ( j=0; j<tam; j++) {
			d1 = x[i][j] - w[i][j];
			if (d1<0) d1 = -d1;
			d += d1;
		}
	}
	return (d);
}

/*---------------------------------------
NF: Function that decreases with time.
Decreases within each round until 0
(if convergence is not reached before)
and also decreases with the number of 
rounds.
---------------------------------------*/

real nf( real no, DATA *D )
{
	 return( no*(1.0-(real)D->t/(real)D->MaxIter)*(1.0-D->fronda*D->tr) );
}


/*---------------------------------------
UPDATE: Updates w cell using x cell. 
This updating depends on the iteration by 
means of nf function
---------------------------------------*/

void  f_update( real e, real ** x, real ** w, DATA *D )
{
	int i, j;
	real factor;

	factor = nf( e, D );

	for ( i=0; i<D->numbases; i++ )
		for ( j=0; j<D->tamalfabeto; j++ )
			 w[i][j] += factor*( x[i][j]-w[i][j] );
}


/*---------------------------------------
BMU: Best Matching Unit
---------------------------------------*/

real BestMU( char modea, DATA *D, int RangoX, int *BMU )
{
	real d, mindist=1000.0f;
	int bmu=0;
	real **x;

	x = D->inputvector[RangoX];


	/** If left descendant of the range is terminal **/

	if ( D->lcells.rango[RangoX]->lef->lef == NULL ) {
		d = distancia( modea, x, D->lcells.rango[RangoX]->lef->sec, D->numbases,
					   D->tamalfabeto );
		if ( d < mindist ) {
			bmu = D->lcells.rango[RangoX]->lef->indlist;
			mindist = d;
		}
	}
	/** If right descendant of the rango es terminal **/
	if ( D->lcells.rango[RangoX]->rig->lef == NULL ) {
		d = distancia( modea, x, D->lcells.rango[RangoX]->rig->sec,
					   D->numbases, D->tamalfabeto );
		if ( d < mindist ) {
			bmu = D->lcells.rango[RangoX]->rig->indlist;
			mindist = d;
		}
	}

	/** increase rsources **/
	D->lcells.recursos[bmu] = D->lcells.recursos[bmu] + mindist;
	D->lcells.rec2[bmu] = D->lcells.rec2[bmu] + sqrt( mindist );

	/** Incrementar el numero de inputs recibidos **/
	D->lcells.inputs[bmu] += 1;

	*BMU = bmu;
	return( mindist );
}


/*---------------------------------------
ADAPTAR (ADAPT): transmission of the interaction
from a updated node to both the ancestor and
sister nodes
---------------------------------------*/

void  adaptar (  real ** x, PCell nodo, DATA *D )
{
	/**  update winning cell **/
	f_update(  D->eb, x, nodo->sec, D);

	if ( nodo->up->update ) {
		/** update upper (ancestor) node **/
		f_update( D->ens, x, nodo->up->sec, D );
		/** update sister node and     **/
		/** continue from the updated  **/
		if ( nodo->up->lef == nodo)    /**  sister = right **/
			f_update( D->enh, x, nodo->up->rig->sec, D );
		else                           /** sister = left **/
			f_update( D->enh, x, nodo->up->lef->sec, D );
	}
}


/*---------------------------------------
DESCRIBE: Description of SOTA tree
---------------------------------------*/

void  describe ( char modea, PCell nodo, DATA *D )
{
	if ( nodo->lef != NULL ) {  /** if not terminal **/
		fprintf( D->fp, "node %d\n", nodo->id );
		fprintf( D->fp, "Dist. to %d = %f\n", nodo->lef->id,
		   distancia( modea, nodo->sec, nodo->lef->sec, D->numbases,D->tamalfabeto) );
		fprintf( D->fp, "Dist. to %d = %f\n", nodo->rig->id,
		   distancia( modea, nodo->sec, nodo->rig->sec, D->numbases,D->tamalfabeto) );
		describe( modea, nodo->lef, D );
		describe( modea, nodo->rig, D );
	}
}

/*---------------------------------------
SAVE SEQUENCE
---------------------------------------*/

void graba_sec( real ** sec, DATA *D )
{
	int i, j;

	if ( D->residuoposible == Base ) {
		for( j=0; j<D->numbases; j++ ) fprintf( D->fp, "%5d ", j );
		fprintf( D->fp, "\n" );
		for( i=0; i<D->tamalfabeto; i++ ) {
			fprintf( D->fp, "%c ", BasePosible[i] );
			for( j=0; j<D->numbases; j++ ) fprintf( D->fp, "%f ", sec[j][i] );
			fprintf( D->fp, "\n" );
		}
	}
	else {
		for( j=0; j<D->numbases; j++ ) fprintf( D->fp, "%5d ", j );
		fprintf( D->fp, "\n" );
		for( i=0; i<D->tamalfabeto; i++ ) {
			fprintf( D->fp, "%c ", AacPosible[i] );
			for( j=0; j<D->numbases; j++ ) fprintf( D->fp, "%f ", sec[j][i] );
			fprintf( D->fp, "\n" );
		}
	}
}


/*---------------------------------------
DESCRIBE NODOS: Description of nodes of
SOTA tree
---------------------------------------*/

void  describe_nodos ( PCell nodo, DATA *D )
{
	if ( nodo->lef != NULL ) {  /**  if not terminal **/
		fprintf( D->fp, "node %d -> %d & %d\n", nodo->id,
						 nodo->lef->id, nodo->rig->id );
		graba_sec( nodo->sec, D );
		describe_nodos( nodo->lef, D );
		describe_nodos( nodo->rig, D );
	}
	else {
		fprintf( D->fp, "node %d\n", nodo->id );
		graba_sec( nodo->sec, D );
	}
}


/*--------------------------------
RECURSOS (RESOURCES): Returns the
index of the cell with greater resources
as well as the resources value
---------------------------------------*/

void recursos( DATA *D  )
{
	int  i;

	D->indice = 0;
	D->TotDist = 0.0f;
	D->MaxRec = 0.0f;

	for ( i=0; i < D->lcells.num; i++ ) {
		if ( D->lcells.inputs[i] > 0 ) {
			/** Sum distance up previous to the change in the resources value **/
			D->TotDist +=  D->lcells.recursos[i];
			/** Decide the procedure to calculate resources **/
				switch (  D->recproc ) {
					case Varianza :
						D->lcells.recursos[i] = D->lcells.rec2[i] -
							sqrt( D->lcells.recursos[i] )/D->lcells.inputs[i];
						break;
					case Media :
						D->lcells. recursos[i] =
							D->lcells.recursos[i]/D->lcells.inputs[i];
						break;
				}
				/** Find the largest resource value **/
				if (D->lcells.recursos[i] > D->MaxRec ) {
					D->MaxRec = D->lcells.recursos[i];
					D->indice = i;
				}
		}
	}
}


/*---------------------------------------
EPOCH : As much iterations as sequences.
It is important to decide wheter a nodo
is duplicated or not.
---------------------------------------*/

real  epoch( char modea, DATA *D )
{
	int i, bmu;
	real error=0;

	/* Initialize inputs density */
	for( i=0; i< D->lcells.num; i++ ) {
		D->lcells.recursos[i] = 0;
		D->lcells.rec2[i] = 0;
		D->lcells.inputs[i] = -1;
	}
	/** Present input **/
	for ( i=0; i<D->numseq; i++ ) {
		++D->t;
		/** find la best matching unit             **/
		/** and increase their resouce value       **/
		/** Here, the network error is accumulated **/
		error += BestMU( modea, D, i, &bmu );

		/** point to the winner **/
		D->inputBMU[i] = bmu;

		/** Update the tree from the winning cell **/
		adaptar( D->inputvector[i], D->lcells.cells[bmu], D );
	}

	recursos( D );
	D->fin = (D->MaxRec <= D->RecFinal);
	return( error );
}


/*---------------------------------------
PERMITIR (ALLOW): Returns true if no
cell was found having no associated inputs
---------------------------------------*/

int permitir ( int n, int *input )
{
	int i, flag=TRUE;

	for ( i=0; i < n; i++ ) {
		if ( input[i] < 0 ) {
			flag=FALSE;
			break;
		}
	}
	return( flag );
}


/*---------------------------------------
INSTANTANEA (Snapshot)
---------------------------------------*/

void  instantanea( ListCells *cel )
{
	int j;

	printf("Nodos\n");

	for ( j=0; j<cel->num; j++ ) {
	 	printf( "%d = node(%d);  ", j, cel->cells[j]->id );
		if ( !(j%5) ) printf("\n");
	}
	printf( "\n" );
}


/*---------------------------------------
DICCIONARIOS. (Codebook)
---------------------------------------*/

void  diccionarios ( DATA *D )
{
	if ( (D->fp=fopen( "codebook.txt", "w" ))==NULL) {
		printf("Can't open codebook.txt file\n");
		exit( 1 );
	}
	fprintf( D->fp, "Codebook vectors\n\n" );
	describe_nodos( D->raiz, D );
	fclose( D->fp );
}


/*---------------------------------------
CICLO (Cicle): Iterates until the network
converges.
---------------------------------------*/

void  ciclo( char modea, DATA *D  )
{
	real error, ErrorAnt;

	D->t = 0;
	D->converge = FALSE;
	ErrorAnt = 10000;
	do {
		/** informar **/
		if ( D->debug ){
			if ( D->t%D->tics == 0 ) printf( "%d\n", D->t );
			else printf(".");
		}

		error = epoch( modea, D );  /** t increases here **/

		/** keep treck of the error **/
		if ( permitir(D->lcells.num, D->lcells.inputs ) ) {
			D->converge = ( (fabs(error-ErrorAnt)/ErrorAnt < D->umbral)
						  || (D->MaxRec <= D->RecFinal) );
			ErrorAnt = error;

		if ( D->debug ) printf( "Error = %f\n", error);
		}
		if( D->fp != NULL ) fprintf( D->fp, "%d %f\n", D->ttot + D->t, error );
		if ( D->debug ) if ( !(D->t%D->tics) ) instantanea( &D->lcells );
	} while(  !D->converge && ( D->t < D->MaxIter) );
}


void allocate_memory ( DATA *D )
{
	int n;
	int flag=0;

	n = D->numseq;

  for( ;; ) {
	D->lcells.cells = ( PCell *) malloc( n*sizeof( PCell ));
	if( D->lcells.cells == NULL ) { flag=1; break; }
	D->lcells.rango = ( PCell *) malloc( n*sizeof( PCell ));
	if( D->lcells.rango == NULL ) { flag=1; break; }
	D->lcells.recursos = ( real *) malloc( n*sizeof( real ));
	if( D->lcells.recursos == NULL ) { flag=1; break; }
	D->lcells.rec2 = ( real *) malloc( n*sizeof( real ));
	if( D->lcells.rec2 == NULL ) { flag=1; break; }
	D->lcells.inputs = ( int *) malloc( n*sizeof( int ));
	if( D->lcells.inputs == NULL ) { flag=1; break; }

	D->inputvector = ( real *** ) malloc( n*sizeof( real **) );
	if( D->inputvector == NULL ) { flag=1; break; }
	D->inputBMU = (int *) malloc( n*sizeof( int ) );
	if( D->inputBMU == NULL ) { flag=1; break; }
	D->nombre = (char **) malloc( n*sizeof( char * ) );
	if( D->nombre == NULL ) { flag=1; break; }
	if( !flag ) break;
  }
	if( flag ) {
		printf("Error: No memory!\n" );
		exit( 1 );
	}
}

void guarda_resultados ( char modea, DATA *D )
{
	int i;

	if ( (D->fp = fopen( D->nameout, "w" ))==NULL ) {
		printf("Can't open %s file\n", D->nameout );
		return;
	}

	fprintf( D->fp, "Results. File %s\n", D->nameout );
	fprintf(D->fp,"The CPU time used is : %.2f minutes\n", cputime_in_minute);
	fprintf( D->fp, "-----------------------------------------\n" );
	fprintf( D->fp, "Convergence : \n" );
	if ( D->fin )  fprintf( D->fp, "Ok!\n" );
	else {
		fprintf( D->fp, "No!\n" );
		if ( permitir(D->lcells.num, D->lcells.inputs) )
			fprintf( D->fp,">>> All nodes were occuped\n" );
		else
			fprintf( D->fp,">>> It was not possible to occupate all the nodes\n");
	}
	fprintf( D->fp, "Iterations Needed = %d\n", D->ttot );
	fprintf( D->fp, "Distance = %f\n", D->TotDist );
	fprintf( D->fp, "MaxIter0 = %d\n", D->MaxIter0 );
	if (modea == '3') fprintf(D->fp, "Number of satasets for analysis = %d\n",D->numbases);
	fprintf( D->fp, "End in %d rounds\n", D->tr );
	fprintf( D->fp, "eb = %f, ens = %f, enh = %f\n",
		D->eb, D->ens, D->enh );
	fprintf( D->fp, "Threshold = %f\n", D->umbral );
	fprintf( D->fp, "Final Recursos = %f\n", D->RecFinal );
	fprintf( D->fp, "No Final = %f.  Slope = -%f\n",
		D->Nfinal, D->fronda );
	fprintf( D->fp, "Input file: %s\n", D->namein );

	fprintf( D->fp, "-----------------------------------------\n" );
	describe( modea, D->raiz, D );
	fprintf( D->fp, "-----------------------------------------\n" );
	for ( i=0; i<D->numseq; i++ ) {
		fprintf( D->fp, "%s goes to %d\n", D->nombre[i],
				  D->lcells.cells[ D->inputBMU[i] ]->id );
	}
	fclose(D->fp);
	D->fp = NULL;
}


/**
==============================================================
MAIN PROGRAM:
==============================================================
**/
/**  Self Organizing Tree Algorithm SOTA  **/
int main( )
{
	int i;
	int NumOrden;
	DATA datos;
	char *paux;
	unsigned long startime,endtime;
	char amode;      /** for input file modes: aligned sequences
						 or a composition matrix  **/
	char *charp;

	clrscr();
	/** Clean data **/
	paux = (char *) &datos;
	for( i=0; i< sizeof(DATA); i++) *paux++ = 0;

	clrscr();
	amode = mainmenu(&datos);

	/** Default values **/
	datos.MaxIter0 = 100;
	datos.umbral = 0.001f;
	datos.Nfinal = 0.5f;
	datos.NumRondas = 0;
	datos.RecFinal = 0.001f;
	datos.eb = 0.1f;
	datos.ens = 0.05f;
	datos.enh = 0.01f;
	datos.tics = 50;

	/** Read number of sequences and residues **/
	switch (amode) {
		case '1':         /** '1' for aligned sequence input   **/
			if( seqinputvals( &datos ) == FALSE ) {
				printf("Error reading input file!\n");
				exit(1);
			}
			datos.inputseqformat = interleaved;
			datos.tamalfabeto = 21;
			break;
		case '2': 	      /** '2' for dipeptide composition input  **/
			if (matinputvals( &datos ) == FALSE ) {
				printf("Error reading input file!\n");
				exit(1);
			}
			datos.tamalfabeto = 20;  /** we know this is a 20*20 matrix  **/
			datos.numbases = 20;
			break;
		case '3' :        /** '3' for distance matrix input  **/
			if (distinputvals (&datos) == FALSE ) {
				printf("Error reading input file!\n");
				exit (1);
			}
			datos.numbases = 1;    /** 1 for one set of distance matrix **/
			break;
	}

	datos.recproc = Media;
	datos.residuoposible = Aminoacido;

	/** initialize the parameters     **/
	argumentos( amode, &datos );

	NumOrden = 3;    /** node's creation number: we start with **/
			 /** a tree with three nodes               **/

	if( datos.debug )
		printf("Date readed %d %d\n", datos.numseq, datos.numbases );

	allocate_memory ( &datos );
	if( datos.debug )
		printf("Memory allocated\n");

	/** Read Sequences **/
	if (amode=='1') seqinputdata(&datos);
	if (amode=='2') matinputdata(&datos);
	if (amode=='3') distinputdata(&datos);

	startime = time (NULL);   /** get system time  **/

	if ( datos.fp != NULL ) {
		printf("Error!!!\n");
		exit(1);
	}

/**	if( datos.namealias[0] != 0 ) {
		if ( (datos.fp = fopen( datos.namealias, "w" ))==NULL ) {
			printf("Error opening wraper file\n");
			exit(1);
		}
		fprintf( datos.fp, "Espia\n" );
	}
	else datos.fp = NULL;
**/

	/** Variables **/
	datos.MaxIter = datos.MaxIter0 * datos.numbases * datos.tamalfabeto;
	if ( datos.NumRondas == 0 || (datos.NumRondas > datos.numseq)  )
		datos.NumRondas = datos.numseq;

	/** decreasing factor for each round: **/
	datos.fronda = (1-datos.Nfinal)/datos.NumRondas;

	inicia_arbol( &datos );
	datos.ttot = 0;
	datos.tr = 0;
	do {  /** NumRondas **/
		datos.tr++;
		printf("\n\n\t\tProcessing Cycle %d ...\n",datos.tr);
		if( datos.debug ) printf( "Round  %d\n", datos.tr );
		/**  Round **/
		ciclo( amode, &datos );
		datos.ttot +=  datos.t;
		if (  !datos.fin && permitir(datos.lcells.num, datos.lcells.inputs) ) {
			duplica_nodo( &NumOrden, &datos);
			if( datos.debug ) {
				printf( "Node %d with resources %f diplicates\n",
					datos.indice, datos.MaxRec );
				instantanea( &datos.lcells );
			}
		}
	} while( (datos.tr < datos.NumRondas) && !datos.fin  );
	fclose( datos.fp );
	datos.fp = NULL;

	if( datos.debug )  printf("\n\nI finished!!! Results:\n\n");

	endtime = time (NULL);          /** get system time again   **/
	cputime_in_minute = difftime(endtime,startime)/60;

	/** resultados **/
	if( datos.nameout[0] != 0 ) guarda_resultados( amode, &datos );

	if( datos.debug ) printf("\n\nCONTINUE!!!\n\n");

	/** save codebooks **/
/**	if( datos.debug ) diccionarios( &datos );   **/
	diccionarios(&datos);
	printf( "done!\n");
	return (1);
}
